#!/usr/bin/bash

# extern_graph,sh -- graph global symbol refs and defs

# ----------------------------------------------------------------
# This file is part of code_graphs Copyright (C) 2011 Elijah 
# Laboratories Inc.  http://www.elilabs.com  ALL RIGHTS RESERVED.
#
# GNU cflow is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# code_graphs is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with code_graphs; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
# ----------------------------------------------------------------


echo '---------------- linkage graphs ----------------'

# default to local mode
MODE=${MODE:-"file:"}
if [ "$MODE" != "file:" ]
then MODE=${MODE}/extern_graph/
fi

# get rid of any old results
rm -rf extern_graph
mkdir -p extern_graph

# start with a clean slate
rm -f *.pdf *.ps *.dot

# list all global defs and refs
nm -C -o -g *.o \
 | awk '{ file = substr($1, 1, index($1, ":")-3) ".c"
          type = $2
          x = index($3, "(")
          if (x == 0) { symbol = $3 }
          else { symbol = substr($3, 1, x-1) }
          print file, type, symbol }' - \
> nm.out

# make dot lines for all defs
awk -v dq='"' '$2 != "U" { print dq $1 dq, "->", dq $3 dq, "[color=red]" }' nm.out > dep.dot

# make dot lines for all refs
awk -v dq='"' '$2 == "U" { print dq $3 dq, "->", dq $1 dq }' nm.out >> dep.dot
cp dep.dot extern_graph.dot
cat extern_graph.dot | grep -v nesting > dep.dot

# make all files cyan hexagons
cat nm.out \
 | awk -v mode=$MODE -v dq='"' '{ print dq $1 dq, "[shape=hexagon,style=filled,fillcolor=lightskyblue,href=" dq mode  $1 ".pdf" dq "];" }' - > dep.sty

# make all functions red trapezoids
cat nm.out \
 | awk -v mode=$MODE -v dq='"' '$2 == "T" { print dq $3 dq, "[shape=invtrapezium,style=filled,fillcolor=red,href=" dq mode  $1 ".pdf" dq "];" }' - >> dep.sty

# make all initialized variables green rectangles
cat nm.out \
 | awk -v mode=$MODE -v dq='"' '$2 == "D" { print dq $3 dq, "[shape=rectangle,style=filled,fillcolor=green,href=" dq mode  $1 ".pdf" dq "];" }' - >> dep.sty

# make all uninitialized variables yellow rectangles
cat nm.out \
 | awk -v mode=$MODE -v dq='"' '$2 == "B" { print dq $3 dq, "[shape=rectangle,style=filled,fillcolor=yellow,href=" dq mode  $1 ".pdf" dq "];" }' - >> dep.sty
cp dep.sty extern_graph.sty
cat extern_graph.sty | grep -v nesting > dep.sty

# as a result of the above, symbols that are still undefined will be white elipses!

# generate the graph
make -f dot_makefile clean > /dev/null
make -f dot_makefile 2>&1 > /dev/null
mv dep.pdf '==ALL==.pdf'
mv dep.ps '==ALL==.ps'
mv dep.in '==ALL==.dot'

# generate all of the subgraphs
./extern_subgraphs.sh

# now make the table of contents
./extern_graph_dir.sh > dir.html

# put in final home
cp *.html *.dot *.ps *.pdf *.gif extern_graph/

# fix up title for browser
cat index.html.skel | sed 's/Dependency Graph Browser/Linkage Graph/' > extern_graph/index.html

