#!/usr/bin/bash

# find_source_nodes.sh -- filter to extract source nodes from a dep list

# ----------------------------------------------------------------
# This file is part of code_graphs Copyright (C) 2011 Elijah 
# Laboratories Inc.  http://www.elilabs.com  ALL RIGHTS RESERVED.
#
# GNU cflow is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# code_graphs is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with code_graphs; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
# ----------------------------------------------------------------


# This script reads a list of dependencies in dot format on stdin and
# outputs on stdout a list of node names that are source nodes --
# nodes that only have arcs emanating from them, but no arcs entering
# into them.  Since the dep list is composed of lines of the form:

#       A -> B

# meaning A precedes B, or B depends on A, then the source nodes are
# all the nodes that are listed on the left hand side, but not on the
# right hand side.


# copy stdin so we can use it multiple times.
cat - \
 | tr -d '";' \
> find_source_nodes.in

# Make a list of LHS nodes.
cat find_source_nodes.in \
 | awk '{ print $1 }' - \
 | sort -u \
> find_source_nodes.lhs

# Make a list of RHS nodes.
cat find_source_nodes.in \
 | awk '{ print $3 }' - \
 | sort -u \
> find_source_nodes.rhs

# Compare the 2 lists to find the source nodes.
join -v 1 find_source_nodes.lhs find_source_nodes.rhs

