#!/usr/bin/bash

# include_graph.sh -- make a graph file #include dependencies

# ----------------------------------------------------------------
# This file is part of code_graphs Copyright (C) 2011 Elijah 
# Laboratories Inc.  http://www.elilabs.com  ALL RIGHTS RESERVED.
#
# GNU cflow is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# code_graphs is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with code_graphs; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
# ----------------------------------------------------------------


echo '---------------- #include graphs ----------------'

# default to local mode
MODE=${MODE:-"file:"}
if [ "$MODE" != "file:" ]
then MODE=${MODE}/include_graph/
fi

# start with a clean slate
rm -f *.pdf *.ps *.dot

# make a list of dependencies
for f in *.[ch]
do awk -v fil=$f -v dq='"' '/^#include/ { print dq fil dq " -> " dq substr($2,2,length($2)-2) dq ";" }' $f
done \
> include_graph.out

# sort the list and purge duplicates
cat include_graph.out | sort -u > include_graph.dot

# make all source nodes red trapezoids
cat include_graph.dot \
 | ./find_source_nodes.sh \
 | awk -v mode=$MODE -v dq='"' '{ print dq $0 dq " [shape=invtrapezium,style=filled,fillcolor=red,href=" dq mode $1 ".pdf" dq "];" }' - \
> include_graph.sty

### FIXME -- CROCK !!! Force *ALL* *.c files to be source nodes
ls *.c \
 | awk -v mode=$MODE  -v dq='"' '{ print dq $0 dq " [shape=invtrapezium,style=filled,fillcolor=red,href=" dq mode $1 ".pdf" dq "];" }' - \
> include_graph.sty

# make all sink nodes green rectangles
cat include_graph.dot \
 | ./find_sink_nodes.sh > sink_nodes.dat
cat sink_nodes.dat \
 | awk -v mode=$MODE  -v dq='"' '{ print dq $0 dq " [shape=rectangle,style=filled,fillcolor=green,href=" dq mode $1 ".pdf" dq "];" }' - \
>> include_graph.sty

# find all system #include <files>
for f in *.[ch]
do awk -v fil=$f -v dq='"' '/^#include/ && /<.+>/ { print $2 }' $f
done \
 | tr -d '<>' \
 | sort -u \
> sys_includes.dat

# find all terminal <sys> nodes
join sys_includes.dat sink_nodes.dat > sys_term_nodes.dat

# find all non-terminal <sys> nodes
join -v 1 sys_includes.dat sys_term_nodes.dat > sys_non_term_nodes.dat

# add color to non-terminal <sys> nodes
cat sys_non_term_nodes.dat \
 | awk -v mode=$MODE  -v dq='"' '{ print dq $0 dq " [style=filled,fillcolor=lightskyblue,href=" dq mode $1 ".pdf" dq "];" }' - \
>> include_graph.sty

# add color to terminal <sys> nodes
cat sys_term_nodes.dat \
 | awk -v mode=$MODE  -v dq='"' '{ print dq $0 dq " [shape=rectangle,style=filled,fillcolor=purple,href=" dq mode $1 ".pdf" dq "];" }' - \
>> include_graph.sty

# make all the rest of the nodes default

# first, make a list of the source and sink nodes
cat include_graph.sty | tr -d '"' | awk '{ print $1 }' - | sort -u > source_and_sink.nodes

# next make a list of all nodes in the graph
cat include_graph.dot | ./all_nodes_in.sh > all.nodes

# now make a list of all non-source/sink nodes
join -v 1 all.nodes source_and_sink.nodes > other.nodes

# make style lines for them too
cat other.nodes \
 | awk -v mode=$MODE  -v dq='"' '{ print dq $0 dq " [style=filled,fillcolor=yellow,href=" dq mode $1 ".pdf" dq "];" }' - \
>> include_graph.sty

# pass the dependency and style files to the makefile
cp include_graph.dot dep.dot
cp include_graph.sty dep.sty

# generate the pdf file for the graph
make -f dot_makefile clean > include_graph_make.err
make -f dot_makefile >> include_graph_make.err

# put the result in the right place
mv dep.pdf '==ALL==.pdf'
mv dep.ps '==ALL==.ps'
mv dep.in '==ALL==.dot'

# generate all of its subgraphs
./include_subgraphs.sh

# now make the table of contents
./include_graph_dir.sh > dir.html

# put in final home
rm -rf include_graph
mkdir -p include_graph
cp *.html *.dot *.ps *.pdf *.gif include_graph/

# fix up title for browser
cat index.html.skel | sed 's/Dependency Graph Browser/Include Graph/' > include_graph/index.html

