#!/usr/bin/perl

# recursive_depends_on.pl <dbase> <node_name> -- filter to extract all
# nodes that recursivly depend on the specified <node_name> in the
# <dbase> file that holds a list of depends-on relationships.

# ----------------------------------------------------------------
# This file is part of code_graphs Copyright (C) 2011 Elijah 
# Laboratories Inc.  http://www.elilabs.com  ALL RIGHTS RESERVED.
#
# GNU cflow is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# code_graphs is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with code_graphs; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
# ----------------------------------------------------------------


# fast iterative perl replacement for recursive_depends_on.sh, which
# was a recursive shell script that ran out of memory and dies on
# call_graph, but worked OK on include_graph.

$dbase = $ARGV[0];		      # the dependency relationship file
$node_name = $ARGV[1];		      # the name of the node to select on
pop(@ARGV);

$true = 1;
$false = 0;

while (<>) {                          # read the dependency relaationships
  chomp;

  ($lhs,$arrow,$rhs) = split;	      # get LHS and RHS from input line

  push(@lhs,$lhs);		      # append to internal arrays
  push(@rhs,$rhs);
  push(@tag,$false);		      # tag is true if line is selected

}

$n = @lhs;			      # number of lines read
$node = $node_name;		      # node we are looking for
$old_node = "";
while ($old_node ne $node) {

  for ($i=0; $i<$n; $i++) {	      # tag all lines with $node on the LHS
    if ($lhs[$i] eq $node) {
      $tag[$i] = $true;
    }
  }

  $node{$node}=true;		      # remember that we've done this node

  $old_node = $node;		      # now find the next node we haven't done yet
  for ($i=0; $i<$n; $i++) {
    if ($tag[$i] == $true) {
      $node = $rhs[$i] unless $node{$rhs[$i]};
    }
  }
}

print "$node_name\n";
for ($i=0; $i<$n; $i++) {
  if ($tag[$i] == $true) {
    print "$lhs[$i]\n";
    print "$rhs[$i]\n";
  }
}

