#!/bin/bash

# show_cycles.sh <node_set> -- read arcs and nodes from <node_set>, generate call_graph/==CYCLES==.*

# ----------------------------------------------------------------
# This file is part of code_graphs Copyright (C) 2011 Elijah 
# Laboratories Inc.  http://www.elilabs.com  ALL RIGHTS RESERVED.
#
# GNU cflow is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# code_graphs is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with code_graphs; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
# ----------------------------------------------------------------


# <node_set> is a list of arcs and nodes in graphviz's dot format.
# This script reads this file and applies Tarjan's algorithm to find
# all the strongly connected components in the input graph.  Each SCC
# is treated separately, as they are disjoint by definition.  Each SCC
# represents nodes in the input graph that are mutually reachable from
# each other.  This is interesting because when the input graph is a
# call graph for a computer program, each of these SCCs represents
# nodes that are called recursively.

# This script finds the SCCs as a set of nodes.  It then generates,
# for each SCC, the subgraph of the input graph that contains those
# nodes and all the arcs in the input graph that connect between those
# nodes; thus the subgraph represents all the functions and calls
# between them that are recursive.

# For each such recursive subgraph, this script highlight the nodes
# and arcs of that subgraph to make them visually distinct when the
# entire call graph is viewed.


# get the input file
node_set=$1

# Eliminate any old cycle graph.
rm -f ==CYCLES==.*

# generate a file for eah SCC
rm -f SCC.*
cat $node_set | ./tarjan.sh | ./split_sccs.pl

# if we do not have any cycles, then skip the rest of this script
if [ ! -e SCC.1 ] ; then exit ; fi

# generate the arcs for each scc
rm -f arcs.*
for f in `ls SCC.*`
do cat $f | sort -u > show_cycles.1

   grep '\->' $node_set | sort -u \
    | join show_cycles.1 - \
    | awk '{ print $3,$2,$1 }' - | sort -u \
    | join show_cycles.1 - \
    | awk '{ print $3,$2,$1 }' | sort -u > `echo $f | sed 's/SCC/arcs/g'`

done

# extract the style lines from the call_graph
grep 'style=' $node_set | sort -u > show_cycles.1

# join these to make the style lines for the cycle graph
# and make the nodes in the cycles pink
cat SCC.* \
 | awk -v dq='"' '{ print dq $1 dq }' - \
 | sort -u \
 | join - show_cycles.1 \
 | sed s/color=yellow/color=pink/g > dep.sty

# make a graph of the cycles
cat arcs.* | sort -u > show_cycles.1

# color the edges red and make them bold
awk '{ print $0,"[color=red,style=bold]" }' show_cycles.1 > dep.dot

# generate the graph
make -f dot_makefile clean > /dev/null
make -f dot_makefile 2>&1 > /dev/null

# save the results
cp dep.ps ==CYCLES==.ps
cp dep.pdf ==CYCLES==.pdf
cp dep.dot ==CYCLES==.dot
cp dep.sty ==CYCLES==.sty
cp dep.in ==CYCLES==.in

